#ifndef PB_UTILS_H
#define PB_UTILS_H

namespace poobear
{
   /*
    * This class is used to represent decimal numbers
    * using fixed point math. 
    */
   class FixedPoint
   {
      public:
         /*
          * Constructs FixedPoint with value val
          * and using pt bits as decimal places
          */
         FixedPoint(int val = 0, int pt = 8);

         /*
          * Constructs FixedPoint with the decimal 
          * number represented by the string, using
          * pt bits as decimal places.
          *
          * Example: FixedPoint x("12.54",10);
          */
         FixedPoint(char*, int pt = 8);

         /*
          * The value this FixedPoint represents 
          * in int and short
          */
         int toInt();
         short toShort;

         /*
          * A bunch of operators usually associated
          * with numbers.
          */
         FixedPoint operator + (FixedPoint);
         FixedPoint operator - (FixedPoint);
         FixedPoint operator * (FixedPoint);
         FixedPoint operator += (FixedPoint);
         FixedPoint operator -= (FixedPoint);
         FixedPoint operator *= (FixedPoint);

         FixedPoint operator + (int);
         FixedPoint operator - (int);
         FixedPoint operator * (int);
         FixedPoint operator += (int);
         FixedPoint operator -= (int);
         FixedPoint operator *= (int);

         bool operator > (FixedPoint);
         bool operator < (FixedPoint);
         bool operator >= (FixedPoint);
         bool operator <= (FixedPoint);

         FixedPoint operator ++ ();
         FixedPoint operator -- ();
         FixedPoint operator - ();
         FixedPoint operator ++ (int);
         FixedPoint operator -- (int);
   };

   /* 
    * A two dimentional vector (as in a
    * mathematical vector, not the array based
    * collection).
    */
   class Vector2D
   {
      public:
         Vector2D(int x = 0, int y = 0);
         Vector2D(FixedPoint, FixedPoint);

         Vector2D operator + (Vector2D);
         Vector2D operator - (Vector2D);
         Vector2D operator += (Vector2D);
         Vector2D operator -= (Vector2D);
   };
};
#endif


